package com.example.demo.messages.api;

import java.util.Map;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.demo.core.api.PageAttributesMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.service.MessageService;

// Контроллер для сущности "Сообщение"
@Controller
@RequestMapping(MessageController.URL)
public class MessageController {
    // URL для доступа к методам контроллера
    public static final String URL = Constants.ADMIN_PREFIX + "/message";

    // Представление для отображения списка сообщений
    private static final String MESSAGE_VIEW = "messages";

    // Атрибут модели для пагинации
    private static final String PAGE_ATTRIBUTE = "page";

    // Бизнес-логика для сущности "Сообщение"
    private final MessageService messageService;

    // Библиотека для преобразования сущности
    private final ModelMapper modelMapper;

    // Конструктор
    public MessageController(MessageService messageService, ModelMapper modelMapper) {
        this.messageService = messageService;
        this.modelMapper = modelMapper;
    }

    // Преобразовать из сущности в DTO
    private MessageDto toDto(MessageEntity entity) {
        return modelMapper.map(entity, MessageDto.class);
    }

    // Получить все элементы
    @GetMapping
    public String getAll(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        final Map<String, Object> attributes = PageAttributesMapper.toAttributes(
                "message",
                messageService.getAll(0L, page, Constants.DEFAULT_PAGE_SIZE), this::toDto);
        model.addAllAttributes(attributes);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return MESSAGE_VIEW;
    }

    // Удалить элемент
    @PostMapping("/delete/{id}")
    public String delete(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        final var messageEntity = messageService.get(id);
        messageService.delete(messageEntity.getUser().getId(), id);
        return Constants.REDIRECT_VIEW + URL;
    }

    // Опубликовать сообщение
    @PostMapping("/publish/{id}")
    public String publish(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        final var messageEntity = messageService.get(id);
        messageEntity.setIsPublished(true);
        messageService.update(messageEntity.getUser().getId(), id, messageEntity);
        return Constants.REDIRECT_VIEW + URL;
    }
}
