package com.example.demo.messages.model;

import java.time.LocalDateTime;
import java.util.Objects;

import com.example.demo.core.model.BaseEntity;
import com.example.demo.users.model.UserEntity;

import jakarta.persistence.Entity;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.Table;
import jakarta.persistence.Column;

// Сущность "Сообщение"
@Entity
@Table(name = "messages")
public class MessageEntity extends BaseEntity {
    // Отправитель сообщения
    @ManyToOne
    @JoinColumn(name = "userId", nullable = false)
    private UserEntity user;

    // Текст сообщения
    @Column(nullable = false)
    private String text;

    // Дата отправки
    @Column(nullable = false)
    private LocalDateTime date;

    // Признак публикации сообщения
    @Column(nullable = false)
    private boolean isPublished;

    // Конструктор по умолчанию
    public MessageEntity() {
    }

    // Конструктор с параметрами для создания объекта
    public MessageEntity(String text, LocalDateTime date, boolean isPublished) {
        this.text = text;
        this.date = date;
        this.isPublished = isPublished;
    }

    // Получить отправителя
    public UserEntity getUser() {
        return user;
    }

    // Установить отправителя
    public void setUser(UserEntity user) {
        this.user = user;
        if (!user.getMessages().contains(this)) {
            user.getMessages().add(this);
        }
    }

    // Получить текст сообщения
    public String getText() {
        return text;
    }

    // Установить текст сообщения
    public void setText(String text) {
        this.text = text;
    }

    // Получить дату отправления
    public LocalDateTime getDate() {
        return date;
    }

    // Установить дату отправления
    public void setDate(LocalDateTime date) {
        this.date = date;
    }

    // Получить признак публикации сообщения
    public boolean getIsPublished() {
        return isPublished;
    }

    // Установить признак публикации сообщения
    public void setIsPublished(boolean isPublished) {
        this.isPublished = isPublished;
    }

    // Получить хэш-код объекта
    @Override
    public int hashCode() {
        return Objects.hash(id, user.getId(), text, date, isPublished);
    }

    // Сравнить объекты
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null || getClass() != obj.getClass())
            return false;
        final MessageEntity other = (MessageEntity) obj;
        return Objects.equals(other.getId(), id)
                && Objects.equals(other.getUser().getId(), user.getId())
                && Objects.equals(other.getText(), text)
                && Objects.equals(other.getDate(), date)
                && Objects.equals(other.getIsPublished(), isPublished);
    }
}
