package com.example.demo.users.api;

import java.util.Objects;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import com.example.demo.core.configuration.Constants;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

// Контроллер для регистрации пользователя
@Controller
@RequestMapping(UserSignupController.URL)
public class UserSignupController {
    // URL для доступа к методам контроллера
    public static final String URL = "/signup";

    // Представление для отображения формы регистрации
    private static final String SIGNUP_VIEW = "signup";

    // Атрибут модели для обработки данных
    private static final String USER_ATTRIBUTE = "user";

    // Бизнес-логика для сущности "Пользователь"
    private final UserService userService;

    // Библиотека для преобразования сущности
    private final ModelMapper modelMapper;

    // Конструктор
    public UserSignupController(
            UserService userService,
            ModelMapper modelMapper) {
        this.userService = userService;
        this.modelMapper = modelMapper;
    }

    // Преобразовать из DTO в сущность
    private UserEntity toEntity(UserSignupDto dto) {
        return modelMapper.map(dto, UserEntity.class);
    }

    // Регистрация пользователя
    @GetMapping
    public String getSignup(Model model) {
        model.addAttribute(USER_ATTRIBUTE, new UserSignupDto());
        return SIGNUP_VIEW;
    }

    // Регистрация пользователя
    @PostMapping
    public String signup(
            @ModelAttribute(name = USER_ATTRIBUTE) @Valid UserSignupDto user,
            BindingResult bindingResult,
            Model model) {
        if (bindingResult.hasErrors()) {
            return SIGNUP_VIEW;
        }
        if (!Objects.equals(user.getPassword(), user.getPasswordConfirm())) {
            bindingResult.rejectValue("password", "signup:passwords", "Пароли не совпадают.");
            model.addAttribute(USER_ATTRIBUTE, user);
            return SIGNUP_VIEW;
        }

        userService.create(toEntity(user));
        return Constants.REDIRECT_VIEW + Constants.LOGIN_URL + "?signup";
    }
}
