package com.example.demo;

import java.time.LocalDateTime;
import java.util.List;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.service.MessageService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

import jakarta.persistence.EntityManager;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class UserMessageServiceTests {
    @Autowired
    private EntityManager entityManager;

    @Autowired
    private UserService userService;

    @Autowired
    private MessageService messageService;


    private UserEntity user1;
    private UserEntity user2;
    private UserEntity user3;

    @BeforeEach
    void createData() {
        removeData();

        user1 = userService.create(new UserEntity("user1", "password", "mail1@gmail.com"));
        user2 = userService.create(new UserEntity("user2", "password", "mail2@gmail.com"));
        user3 = userService.create(new UserEntity("user3", "password", "mail3@gmail.com"));

        final var messages1 = List.of(
			new MessageEntity("message1", LocalDateTime.now(), false),
			new MessageEntity("message2", LocalDateTime.now(), false),
			new MessageEntity("message3", LocalDateTime.now(), false)
		);
		messages1.forEach(message -> messageService.create(user1.getId(), message));

        final var messages2 = List.of(
			new MessageEntity("Message4", LocalDateTime.now(), false),
			new MessageEntity("Message5", LocalDateTime.now(), false)
		);
		messages2.forEach(message -> messageService.create(user2.getId(), message));
    }

    @AfterEach
    void removeData() {
        userService.deleteAll();
    }

    @Test
    @Order(1)
    void createTest() {
        Assertions.assertEquals(5, messageService.getAll(0L).size());
    }

    @Test
    @Order(2)
    void orderFilterTest() {
        Assertions.assertEquals(3, messageService.getAll(user1.getId()).size());
        Assertions.assertEquals(2, messageService.getAll(user2.getId()).size());
        Assertions.assertEquals(0, messageService.getAll(user3.getId()).size());
    }

    @Test
    @Order(3)
    void userCascadeDeleteTest() {
        userService.delete(user1.getId());
        final var messages = entityManager.createQuery(
                "select count(o) from MessageEntity o where o.user.id = :userId");
        messages.setParameter("userId", user1.getId());
        Assertions.assertEquals(0, messages.getFirstResult());
    }
}
